package main

import (
	"flag"
	"fmt"
	"log"
)

type CommandFunc func(tbm *TBM, args []string) error

var (
	serial_device   = flag.String("serial-device", "/dev/ttyUSB0", "Serial device to use")
	serial_baudrate = flag.Int("serial-baud", 9600, "Serial baud rate")

	commands = map[string]CommandFunc{
		"ls": cmd_ls,
		"cat": cmd_cat,
		"stat": cmd_stat,
		"cp": cmd_cp,
		"rm": cmd_rm,
		"mkdir": cmd_mkdir,
		"rmdir": cmd_rmdir,
		"date": cmd_date,
		"set-date": cmd_set_date,
		"time": cmd_time,
	}
)

func cmd_generic(tbm *TBM, command string, args []string) error {
	cmd := &Command{command, args}
	tbm.Commands <- cmd
	res := <-tbm.Results

	if res.errcode[0] != '0' {
		return fmt.Errorf("Error: %s", res.errcode)
	}
	fmt.Printf("%s", res.output)

	return nil
}

func cmd_ls(tbm *TBM, args []string) error {
	return cmd_generic(tbm, "mufs ls", args)
}

func cmd_cat(tbm *TBM, args []string) error {
	return cmd_generic(tbm, "mufs cat", args)
}

func cmd_stat(tbm *TBM, args []string) error {
	return cmd_generic(tbm, "mufs stat", args)
}

func cmd_cp(tbm *TBM, args []string) error {
	return cmd_generic(tbm, "mufs cp", args)
}

func cmd_rm(tbm *TBM, args []string) error {
	return cmd_generic(tbm, "mufs rm", args)
}

func cmd_mkdir(tbm *TBM, args []string) error {
	return cmd_generic(tbm, "mufs mkdir", args)
}

func cmd_rmdir(tbm *TBM, args []string) error {
	return cmd_generic(tbm, "mufs rmdir", args)
}

func cmd_date(tbm *TBM, args []string) error {
	return cmd_generic(tbm, "date", args)
}

func cmd_set_date(tbm *TBM, args []string) error {
	return cmd_generic(tbm, "set-date", args)
}

func cmd_time(tbm *TBM, args []string) error {
	return cmd_generic(tbm, "time", args)
}

func main() {
	flag.Parse()

	args := flag.Args()
	if len(args) == 0 {
		log.Fatalf("Please specify at least one command")
	}
	command := args[0]

	command_function, ok := commands[command]

	if !ok {
		log.Fatalf("Unknown command: %s", command)
	}

	t := &TBM{}
	err := t.Connect(*serial_device, *serial_baudrate)
	if err != nil {
		log.Fatal(err)
	}
	t.Handle()
	defer t.Close()

	err = command_function(t, args[1:])
	if err != nil {
		log.Fatal(err)
	}
}
